# Copyright (C) 2022 The Android Open Source Project
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import argparse
import shlex


def ParseSignerArgs(args):
  if args is None:
    return None
  return shlex.split(args)


def AddSigningArgumentParse(parser: argparse.ArgumentParser):
  parser.add_argument('--package_key', type=str,
                      help='Paths to private key for signing payload')
  parser.add_argument('--search_path', '--path', type=str,
                      help='Search path for framework/signapk.jar')
  parser.add_argument('--payload_signer', type=str,
                      help='Path to custom payload signer')
  parser.add_argument('--payload_signer_args', type=ParseSignerArgs,
                      help='Arguments for payload signer if necessary')
  parser.add_argument('--payload_signer_maximum_signature_size', type=str,
                      help='Maximum signature size (in bytes) that would be '
                      'generated by the given payload signer')
  parser.add_argument('--private_key_suffix', type=str,
                      help='Suffix to be appended to package_key path', default=".pk8")
